/*****************************************************************************
 *  File: reconfig_utils.h
 *
 *  This file contains the constants, structure definitions, and funtion
 *  prototypes for the application selector.
 *
 ****************************************************************************/

#ifndef APP_SELECTOR_H_
#define APP_SELECTOR_H_

#include "sys/alt_flash_dev.h"

//#define DEBUG_MODE 1 // debugging help

#define ALT_VIDEO_DISPLAY_COLS 800
#define ALT_VIDEO_DISPLAY_ROWS 480
#define ALT_VIDEO_DISPLAY_COLOR_DEPTH 32
#define ALT_VIDEO_DISPLAY_SGDMA_NAME LCD_SGDMA_NAME

#define NUM_VIDEO_FRAMES 2

#define FLASH_SECTOR_SIZE            0x20000	//128KB
#define FLASH_BOOT_BLOCK_SECTOR_SIZE 0x08000	//32KB
#define HW_IMAGE_SIZE                0x0060A78F	// AIIGX 0x0060A78F (pfl erases)
#define SECTORS_FOR_HW_IMAGE         ((HW_IMAGE_SIZE / FLASH_SECTOR_SIZE) + 1) //
#define BYTES_FOR_HW_IMAGE           (SECTORS_FOR_HW_IMAGE * FLASH_SECTOR_SIZE)
#define BOARD_ID 					 0x15     // used for board ID in flash

/****************************************************************
 AIIGX Flash Memory Map

 BLOCK		 START ADDRESS  END ADDRESS	SIZE
 --------------------------------------------------
 Page_0		 0x00020000		0x0062A78F	0x0060A78F
 Page_1		 0x00640000		0x00C4A78F	0x00C1A78F
 OPTION BITS 0x00018000		0x00018080

 Section                 Size  Address
 -----------------------------------------------------
 User Reset Vector       32KB  0x00000000 - 0x00007FFF
 Ethernet Option Bits    32KB  0x00008000 - 0x0000FFFF
 Reserved                32KB  0x00010000 - 0x00017FFF
 PFL Option Bits         32KB  0x00018000 - 0x0001FFFF
 Factory Hardware        ~6MB  0x00020000 - 0x0062FFFF
 User1 Hardware          ~6MB  0x00640000 - 0x00C5FFFF
 User2 Hardware          ~6MB  0x00C60000 - 0x0126FFFF
 Unused Block            ~6MB  0x01280000 - 0x0181FFFF
 zipfs (html content)     8MB  0x01820000 - 0x0201FFFF
 ----------------------------------------------------
 Factory Software         8MB  0x02020000 - 0x0281FFFF
 User Software           24MB  0x02820000 - 0x03FCFFFF
 Top boot blocks        128KB  0x03FD0000 - 0x03FFFFFF

****************************************************************/

#define FACTORY_HW_IMAGE_PAGE        0
#define FACTORY_HW_IMAGE_OFFSET      0x00020000
#define FACTORY_SW_IMAGE_OFFSET      0x02020000

#define USER1_HW_IMAGE_PAGE          1
#define USER1_HW_IMAGE_OFFSET        0x00640000
#define USER2_HW_IMAGE_PAGE          2
#define USER2_HW_IMAGE_OFFSET        0x00C60000
#define USER_SW_IMAGE_OFFSET         0x02820000

#define RO_ZIPFS_OFFSET              0x01820000
#define ETHERNET_OPTION_BITS         0x00008000
#define PFL_OPTION_BIT_OFFSET        0x00018000

#define SREC_BUFFER_SIZE             0x00020000

// These are the AIIGX 32bit masks for the MAXII reconfiguation 32bit registers:
#define MAXII_CLEAR_PSR_MASK         0xFFFFFFFE // 1110 | 0
#define MAXII_CLEAR_PSO_MASK         0xFFFFFFFD // 1101 | 1
#define MAXII_FORCE_RECONFIG_MASK    0x0000000B // 1011 | B
#define MAXII_CLEAR_PSS_MASK         0xFFFFFFF7 // 0111 | 3

#define MAXII_RSU_REG  			0x8 // MAXII_REG2 is 0x8, was 0x6

#define PAGE_USER				1	// NIOS_PIO_PAGE_SELECT_BASE
#define SRST					0	// NIOS_PIO_RECONFIGURE_N_BASE

#define USE_HAL_FLASH_DRIVER            0
#define USE_INTEL_P30_FLASH_DRIVER      1

// Error Codes
#define SUCCESS                      0
#define FAILURE                     -1
#define ERROR_FAT_FAILED_TO_MOUNT   -2
#define ERROR_FAT_COULD_NOT_FIND    -3
#define ERROR_FAT_READ_ZERO_BYTES   -4
#define ERROR_SREC_HAS_NO_TIMESTAMP -5
#define ERROR_COULD_NOT_ALLOCATE    -6

// Filetypes
#define FLASH                        1
#define RBF                          2

#define FILE_BUFFER_SIZE FLASH_SECTOR_SIZE

int ReconfigFPGA( unsigned int hw_flash_page );
void InitializePFLOptionBits( unsigned int option_bit_offset );

#endif /*APP_SELECTOR_H_*/
